      module odf_tables

!.... NOTE: THE WAVELENGTHS HERE ARE BOUNDARY WAVELENGTHS *BETWEEN* THE 
!....       ODF INTERVALS, AND THEY ARE *NOT* THE SAME AS THE 
!....       WAVELENGTHS DEFINED IN MODULE_FREQ_SET, WHICH ARE AT THE 
!....       CENTER OF EACH ODF INTERVAL

!.... 2012 OCT - CHANGED w_odf TO odf_wave
!....          - CHANGED wt_odf TO odf_wt
!.... 2009 JUN - CHANGED data STATEMENTS TO REMOVE LOOP
!.... 2007 SEP - CREATE SEPARATE P AND T TABLES FOR KURUCZ AND CASTELLI

      use physical_constants, only: tenln
      use var_types

      implicit none

!------------------------------ CONSTANTS ------------------------------

      integer(in_type), parameter :: max_odf = 1221
      integer(in_type), parameter :: np_odf_c = 25   ! # CASTELLI PRES
      integer(in_type), parameter :: np_odf_k = 21   ! # KURUCZ PRES
      integer(in_type), parameter :: nt_odf_c = 57   ! # CASTELLI TEMP
      integer(in_type), parameter :: nt_odf_k = 56   ! # KURUCZ TEMP

!.... CASTELLI PRESSURE VALUES - 4 MORE THAN KURUCZ

      real(re_type), parameter :: p_odf_c(np_odf_c) = [
     &  -4.0d0, -3.5d0, -3.0d0, -2.5d0,
     &  -2.0d0, -1.5d0, -1.0d0, -0.5d0, 0.0d0,
     &   0.5d0,  1.0d0,  1.5d0,  2.0d0, 2.5d0,
     &   3.0d0,  3.5d0,  4.0d0,  4.5d0, 5.0d0,
     &   5.5d0,  6.0d0,  6.5d0,  7.0d0, 7.5d0,
     &   8.0d0 ]

!.... KURUCZ PRESSURE VALUES

      real(re_type), parameter :: p_odf_k(np_odf_k) = [
     &  -2.0d0, -1.5d0, -1.0d0, -0.5d0, 0.0d0,
     &   0.5d0,  1.0d0,  1.5d0,  2.0d0, 2.5d0,
     &   3.0d0,  3.5d0,  4.0d0,  4.5d0, 5.0d0,
     &   5.5d0,  6.0d0,  6.5d0,  7.0d0, 7.5d0,
     &   8.0d0 ]

!.... CASTELLI TEMP VALUES - 1 MORE THAN KURUCZ

      real(re_type), parameter :: t_odf_c(nt_odf_c) = [
     &  3.30d0,
     &  3.32d0, 3.34d0, 3.36d0, 3.38d0, 3.40d0,
     &  3.42d0, 3.44d0, 3.46d0, 3.48d0, 3.50d0,
     &  3.52d0, 3.54d0, 3.56d0, 3.58d0, 3.60d0,
     &  3.62d0, 3.64d0, 3.66d0, 3.68d0, 3.70d0,
     &  3.73d0, 3.76d0, 3.79d0, 3.82d0, 3.85d0,
     &  3.88d0, 3.91d0, 3.94d0, 3.97d0, 4.00d0,
     &  4.05d0, 4.10d0, 4.15d0, 4.20d0, 4.25d0,
     &  4.30d0, 4.35d0, 4.40d0, 4.45d0, 4.50d0,
     &  4.55d0, 4.60d0, 4.65d0, 4.70d0, 4.75d0,
     &  4.80d0, 4.85d0, 4.90d0, 4.95d0, 5.00d0,
     &  5.05d0, 5.10d0, 5.15d0, 5.20d0, 5.25d0,
     &  5.30d0 ]

!.... KURUCZ TEMP VALUES

      real(re_type), parameter :: t_odf_k(nt_odf_k) = [
     &  3.32d0, 3.34d0, 3.36d0, 3.38d0, 3.40d0,
     &  3.42d0, 3.44d0, 3.46d0, 3.48d0, 3.50d0,
     &  3.52d0, 3.54d0, 3.56d0, 3.58d0, 3.60d0,
     &  3.62d0, 3.64d0, 3.66d0, 3.68d0, 3.70d0,
     &  3.73d0, 3.76d0, 3.79d0, 3.82d0, 3.85d0,
     &  3.88d0, 3.91d0, 3.94d0, 3.97d0, 4.00d0,
     &  4.05d0, 4.10d0, 4.15d0, 4.20d0, 4.25d0,
     &  4.30d0, 4.35d0, 4.40d0, 4.45d0, 4.50d0,
     &  4.55d0, 4.60d0, 4.65d0, 4.70d0, 4.75d0,
     &  4.80d0, 4.85d0, 4.90d0, 4.95d0, 5.00d0,
     &  5.05d0, 5.10d0, 5.15d0, 5.20d0, 5.25d0,
     &  5.30d0 ]

!.... TURBULENT VELOCITIES IN CM/S

      real(re_type), parameter :: v_odf(5) = [
     &  0.0d0, 1.0d5, 2.0d5, 4.0d5, 8.0d5 ]

!.... ODF STEP WEIGHTS

      real(re_type), parameter :: odf_wt(12) = [
     &   1.0d0 / 60.0d0, 2.0d0 / 60.0d0, 3.0d0 / 60.0d0,
     &   6.0d0 / 60.0d0, 6.0d0 / 60.0d0, 6.0d0 / 60.0d0,
     &   6.0d0 / 60.0d0, 6.0d0 / 60.0d0, 6.0d0 / 60.0d0,
     &   6.0d0 / 60.0d0, 6.0d0 / 60.0d0, 6.0d0 / 60.0d0 ]

!!!!  real(re_type), parameter :: ttenln = tenln * 0.001d0 ! NOT USED ?

!------------------------------ VARIABLES ------------------------------

      integer(in_type) :: np_odf
      integer(in_type) :: nt_odf
      integer(in_type) :: nw_odf

      real(re_type) :: odf_wave(max_odf)
      real(re_type) :: odf_wbig(329)
      real(re_type) :: odf_wlit(1213)
      real(re_type) :: p_odf(np_odf_c) ! USE CASTELLI DIM
      real(re_type) :: t_odf(nt_odf_c) ! USE CASTELLI DIM

!--------------------------- INITIALIZATION ----------------------------

      data odf_wbig(1:329) /
     &     8.97666,    9.20000,    9.50000,    9.71730,    9.81590,
     &    10.10901,   10.30000,   10.46451,   10.65000,   10.88545,
     &    11.20000,   11.60000,   11.95562,   12.30000,   12.66565,
     &    12.74676,   12.93504,   13.16033,   13.31723,   13.46700,
     &    13.86253,   14.10462,   14.54782,   14.90000,   15.30000,
     &    15.74874,   16.00346,   16.40000,   16.80000,   17.25088,
     &    17.43258,   17.93660,   18.10114,   18.24438,   18.97796,
     &    19.22422,   19.55098,   20.11987,   20.23592,   20.80000,
     &    21.30000,   21.94783,   22.01983,   22.58070,   22.78377,
     &    23.20000,   23.65221,   24.30000,   25.00744,   25.46614,
     &    25.89239,   26.14316,   26.60000,   27.10000,   27.60000,
     &    28.09263,   28.65871,   29.30000,   29.95849,   30.26260,
     &    31.30000,   32.30000,   33.30000,   34.30000,   35.27931,
     &    36.14100,   37.00000,   38.01600,   38.96642,   40.00000,
     &    41.00000,   41.87992,   42.50000,   43.50000,   44.73260,
     &    45.50000,   46.50000,   47.50000,   48.50178,   49.50000,
     &    50.42590,   50.87630,   52.00000,   54.00000,   56.00000,
     &    57.40614,   59.50000,   61.50000,   63.50000,   65.53786,
     &    67.00000,   69.00000,   71.00000,   72.24011,   74.00000,
     &    76.00000,   78.00000,   80.00000,   82.00000,   84.00000,
     &    86.0,       88.0,       90.0,       91.17535,   94.0,
     &    98.0,      102.0,      106.0,      110.03056,  113.0,
     &   116.0,      120.0,      123.92928,  128.0,      132.0,
     &   136.0,      140.0,      144.43391,  148.0,      151.43485,
     &   157.0,      162.15072,  167.40282,  171.0,      175.0,
     &   180.0,      184.0,      188.0,      193.0,      197.46990,
     &   202.0,      207.13210,  210.0,      215.0,      220.0,
     &   225.0,      230.0,      235.0,      240.0,      245.0,
     &   251.12621,  255.0,      260.0,      265.0,      270.0,
     &   275.0,      280.0,      285.0,      290.0,      300.0,
     &   310.0,      320.0,      330.0,      340.0,      350.0,
     &   360.0,      364.70183,  370.0,      380.0,      390.0,
     &   400.0,      410.0,      420.0,      430.0,      450.0,
     &   460.0,      470.0,      480.0,      490.0,      500.0,
     &   510.0,      520.0,      530.0,      540.0,      550.0,
     &   560.0,      570.0,      580.0,      590.0,      600.0,
     &   610.0,      620.0,      630.0,      640.0,      650.0,
     &   660.0,      670.0,      680.0,      690.0,      700.0,
     &   710.0,      720.0,      730.0,      740.0,      750.0,
     &   760.0,      770.0,      780.0,      790.0,      800.0,
     &   810.0,      820.58271,  830.0,      840.0,      850.0,
     &   860.0,      870.0,      880.0,      890.0,      900.0,
     &   910.0,      920.0,      930.0,      940.0,      950.0,
     &   960.0,      970.0,      980.0,      990.0,     1000.0,
     &  1025.0,     1050.0,     1075.0,     1100.0,     1125.0,
     &  1150.0,     1175.0,     1200.0,     1225.0,     1250.0,
     &  1275.0,     1300.0,     1325.0,     1350.0,     1375.0,
     &  1400.0,     1425.0,     1458.81670, 1475.0,     1500.0,
     &  1525.0,     1550.0,     1575.0,     1600.0,     1640.0,
     &  1680.0,     1720.0,     1760.0,     1800.0,     1840.0,
     &  1880.0,     1920.0,     1960.0,     2000.0,     2050.0,
     &  2100.0,     2150.0,     2200.0,     2250.0,     2279.40330,
     &  2300.0,     2350.0,     2400.0,     2450.0,     2500.0,
     &  2550.0,     2600.0,     2650.0,     2700.0,     2750.0,
     &  2800.0,     2850.0,     2900.0,     2950.0,     3000.0,
     &  3050.0,     3100.0,     3150.0,     3200.0,     3282.34320,
     &  3400.0,     3500.0,     3600.0,     3700.0,     3800.0,
     &  3900.0,     4000.0,     4100.0,     4200.0,     4300.0,
     &  4400.0,     4500.0,     4600.0,     4700.0,     4800.0,
     &  4900.0,     5000.0,     5100.0,     5200.0,     5300.0,
     &  5400.0,     5500.0,     5600.0,     5700.0,     5800.0,
     &  5900.0,     6000.0,     6100.0,     6200.0,     6300.0,
     &  6400.0,     6600.0,     6800.0,     7000.0,     7200.0,
     &  7400.0,     7600.0,     7800.0,     8000.0,     8200.0,
     &  8400.0,     8600.0,     8800.0,     9000.0,     9200.0,
     &  9400.0,     9600.0,     9800.0, 10000000.0/

      data odf_wlit(1:1213) /
     &     8.97666,    9.2,        9.5,        9.71730,    9.81590,
     &    10.10901,   10.3,       10.46451,   10.65000,   10.88545,
     &    11.2,       11.6,       11.95562,   12.3,       12.66565,
     &    12.74676,   12.93504,   13.16033,   13.31723,   13.46700,
     &    13.86253,   14.10462,   14.54782,   14.9,       15.3,
     &    15.74874,   16.00346,   16.4,       16.8,       17.25088,
     &    17.43258,   17.93660,   18.10114,   18.24438,   18.97796,
     &    19.22422,   19.55098,   20.11987,   20.23592,   20.8,
     &    21.3,       21.94783,   22.01983,   22.58070,   22.78377,
     &    23.2,       23.65221,   24.3,       25.00744,   25.46614,
     &    25.89239,   26.14316,   26.6,       27.1,       27.6,
     &    28.09263,   28.65871,   29.3,       29.95849,   30.26260,
     &    31.3,       32.3,       33.3,       34.3,       35.27931,
     &    36.14100,   37.0,       38.01600,   38.96642,   40.0,
     &    41.0,       41.87992,   42.5,       43.5,       44.73260,
     &    45.5,       46.5,       47.5,       48.50178,   49.5,
     &    50.42590,   50.87630,   51.5,       52.5,       53.5,
     &    54.5,       55.5,       56.5,       57.40614,   58.5,
     &    59.5,       60.5,       61.5,       62.5,       63.5,
     &    64.5,       65.53786,   66.0,       67.0,       68.0,
     &    69.0,       70.0,       71.0,       72.24011,   73.0,
     &    74.0,       75.0,       76.0,       77.0,       78.0, 
     &    79.0,       80.0,       81.0,       82.0,       83.0,
     &    84.0,       85.0,       86.0,       87.0,       88.0, 
     &    89.0,       90.0,       91.17535,   92.0,       93.0,
     &    94.0,       95.0,       96.0,       97.0,       98.0, 
     &    99.0,      100.0,      101.0,      102.0,      103.0,
     &   104.0,      105.0,      106.0,      107.0,      108.0, 
     &   109.0,      110.03056,  111.0,      112.0,      113.0,
     &   114.0,      115.0,      116.0,      117.0,      118.0,
     &   119.0,      120.0,      121.0,      122.0,      123.0,
     &   123.92928,  125.0,      126.0,      127.0,      128.0, 
     &   129.0,      130.0,      131.0,      132.0,      133.0,
     &   134.0,      135.0,      136.0,      137.0,      138.0, 
     &   139.0,      140.0,      141.0,      142.0,      143.0,
     &   144.0,      144.43391,  145.0,      146.0,      147.0,
     &   148.0,      149.0,      150.0,      151.0,      151.43485,
     &   152.0,      153.0,      154.0,      155.0,      156.0, 
     &   157.0,      158.0,      159.0,      160.0,      161.0,
     &   162.15072,  163.0,      164.0,      165.0,      166.0,
     &   167.0,      167.40282,  168.0,      169.0,      170.0,
     &   171.0,      172.0,      173.0,      174.0,      175.0, 
     &   176.0,      177.0,      178.0,      179.0,      180.0,
     &   181.0,      182.0,      183.0,      184.0,      185.0, 
     &   186.0,      187.0,      188.0,      189.0,      190.0,
     &   191.0,      192.0,      193.0,      194.0,      195.0,
     &   196.0,      197.46990,  197.81149,  199.0,      200.0,
     &   201.0,      202.0,      203.0,      204.0,      205.0,
     &   206.0,      207.13210,  207.61400,  208.0,      209.0,
     &   210.0,      211.0,      212.0,      213.0,      214.0,
     &   215.0,      216.0,      217.0,      218.0,      219.0,
     &   220.0,      221.0,      222.0,      223.0,      224.0, 
     &   225.0,      226.0,      227.0,      228.0,      229.0,
     &   230.0,      231.0,      232.0,      233.0,      234.0, 
     &   235.0,      236.0,      237.0,      238.0,      239.0,
     &   240.0,      241.0,      242.0,      243.0,      244.0, 
     &   245.0,      246.0,      247.0,      248.0,      249.0,
     &   250.0,      251.12621,  251.51005,  252.0,      253.0,
     &   254.0,      255.0,      256.0,      257.0,      258.0,
     &   259.0,      260.0,      261.0,      262.0,      263.0,
     &   264.0,      265.0,      266.0,      267.0,      268.0,
     &   269.0,      270.0,      271.0,      272.0,      273.0, 
     &   274.0,      275.0,      276.0,      277.0,      278.0,
     &   279.0,      280.0,      281.0,      282.0,      283.0, 
     &   284.0,      285.0,      286.0,      287.0,      288.0,
     &   289.0,      290.0,      292.0,      294.0,      296.0, 
     &   298.0,      300.0,      302.0,      304.0,      306.0,
     &   308.0,      310.0,      312.0,      314.0,      316.0, 
     &   318.0,      320.0,      322.0,      324.0,      326.0,
     &   328.0,      330.0,      332.0,      334.0,      336.0,
     &   338.0,      340.0,      342.0,      344.0,      346.0,
     &   348.0,      350.0,      352.0,      354.0,      356.0,     
     &   358.0,      360.0,      362.0,      364.0,      364.70183,
     &   366.0,      368.0,      370.0,      372.0,      374.0,     
     &   376.0,      378.0,      380.0,      382.0,      384.0,
     &   386.0,      388.0,      390.0,      392.0,      394.0,     
     &   396.0,      398.0,      400.0,      402.0,      404.0,
     &   406.0,      408.0,      410.0,      412.0,      414.0,     
     &   416.0,      418.0,      420.0,      422.0,      424.0,
     &   426.0,      428.0,      430.0,      432.0,      434.0,
     &   436.0,      438.0,      440.0,      442.0,      444.0,
     &   446.0,      448.0,      450.0,      452.0,      454.0,     
     &   456.0,      458.0,      460.0,      462.0,      464.0,
     &   466.0,      468.0,      470.0,      472.0,      474.0,     
     &   476.0,      478.0,      480.0,      482.0,      484.0,
     &   486.0,      488.0,      490.0,      492.0,      494.0,     
     &   496.0,      498.0,      500.0,      502.0,      504.0,
     &   506.0,      508.0,      510.0,      512.0,      514.0,     
     &   516.0,      518.0,      520.0,      522.0,      524.0,
     &   526.0,      528.0,      530.0,      532.0,      534.0,
     &   536.0,      538.0,      540.0,      542.0,      544.0,
     &   546.0,      548.0,      550.0,      552.0,      554.0,     
     &   556.0,      558.0,      560.0,      562.0,      564.0,
     &   566.0,      568.0,      570.0,      572.0,      574.0,     
     &   576.0,      578.0,      580.0,      582.0,      584.0,
     &   586.0,      588.0,      590.0,      592.0,      594.0,     
     &   596.0,      598.0,      600.0,      602.0,      604.0,
     &   606.0,      608.0,      610.0,      612.0,      614.0,     
     &   616.0,      618.0,      620.0,      622.0,      624.0,
     &   626.0,      628.0,      630.0,      632.0,      634.0,
     &   636.0,      638.0,      640.0,      642.0,      644.0,
     &   646.0,      648.0,      650.0,      652.0,      654.0,     
     &   656.0,      658.0,      660.0,      662.0,      664.0,
     &   666.0,      668.0,      670.0,      672.0,      674.0,     
     &   676.0,      678.0,      680.0,      682.0,      684.0,
     &   686.0,      688.0,      690.0,      692.0,      694.0,     
     &   696.0,      698.0,      700.0,      702.0,      704.0,
     &   706.0,      708.0,      710.0,      712.0,      714.0,     
     &   716.0,      718.0,      720.0,      722.0,      724.0,
     &   726.0,      728.0,      730.0,      732.0,      734.0,
     &   736.0,      738.0,      740.0,      742.0,      744.0,
     &   746.0,      748.0,      750.0,      752.0,      754.0,     
     &   756.0,      758.0,      760.0,      762.0,      764.0,
     &   766.0,      768.0,      770.0,      772.0,      774.0,     
     &   776.0,      778.0,      780.0,      782.0,      784.0,
     &   786.0,      788.0,      790.0,      792.0,      794.0,     
     &   796.0,      798.0,      800.0,      802.0,      804.0,
     &   806.0,      808.0,      810.0,      812.0,      814.0,     
     &   816.0,      818.0,      820.58271,  822.0,      824.0,
     &   826.0,      828.0,      830.0,      832.0,      834.0,
     &   836.0,      838.0,      840.0,      842.0,      844.0,
     &   846.0,      848.0,      850.0,      852.0,      854.0,     
     &   856.0,      858.0,      860.0,      862.0,      864.0,
     &   866.0,      868.0,      870.0,      872.0,      874.0,     
     &   876.0,      878.0,      880.0,      882.0,      884.0,
     &   886.0,      888.0,      890.0,      892.0,      894.0,     
     &   896.0,      898.0,      900.0,      902.0,      904.0,
     &   906.0,      908.0,      910.0,      912.0,      914.0,     
     &   916.0,      918.0,      920.0,      922.0,      924.0,
     &   926.0,      928.0,      930.0,      932.0,      934.0,
     &   936.0,      938.0,      940.0,      942.0,      944.0,
     &   946.0,      948.0,      950.0,      952.0,      954.0,     
     &   956.0,      958.0,      960.0,      962.0,      964.0,
     &   966.0,      968.0,      970.0,      972.0,      974.0,     
     &   976.0,      978.0,      980.0,      982.0,      984.0,
     &   986.0,      988.0,      990.0,      992.0,      994.0,     
     &   996.0,      998.0,     1000.0,     1005.0,     1010.0,
     &  1015.0,     1020.0,     1025.0,     1030.0,     1035.0, 
     &  1040.0,     1045.0,     1050.0,     1055.0,     1060.0, 
     &  1065.0,     1070.0,     1075.0,     1080.0,     1085.0,
     &  1090.0,     1095.0,     1100.0,     1105.0,     1110.0, 
     &  1115.0,     1120.0,     1125.0,     1130.0,     1135.0, 
     &  1140.0,     1145.0,     1150.0,     1155.0,     1160.0, 
     &  1165.0,     1170.0,     1175.0,     1180.0,     1185.0, 
     &  1190.0,     1195.0,     1200.0,     1205.0,     1210.0, 
     &  1215.0,     1220.0,     1225.0,     1230.0,     1235.0, 
     &  1240.0,     1245.0,     1250.0,     1255.0,     1260.0, 
     &  1265.0,     1270.0,     1275.0,     1280.0,     1285.0, 
     &  1290.0,     1295.0,     1300.0,     1305.0,     1310.0, 
     &  1315.0,     1320.0,     1325.0,     1330.0,     1335.0,
     &  1340.0,     1345.0,     1350.0,     1355.0,     1360.0, 
     &  1365.0,     1370.0,     1375.0,     1380.0,     1385.0, 
     &  1390.0,     1395.0,     1400.0,     1405.0,     1410.0, 
     &  1415.0,     1420.0,     1425.0,     1430.0,     1435.0, 
     &  1440.0,     1445.0,     1450.0,     1455.0,     1458.81670, 
     &  1465.0,     1470.0,     1475.0,     1480.0,     1485.0, 
     &  1490.0,     1495.0,     1500.0,     1505.0,     1510.0, 
     &  1515.0,     1520.0,     1525.0,     1530.0,     1535.0, 
     &  1540.0,     1545.0,     1550.0,     1555.0,     1560.0, 
     &  1565.0,     1570.0,     1575.0,     1580.0,     1585.0,
     &  1590.0,     1595.0,     1600.0,     1610.0,     1620.0,
     &  1630.0,     1640.0,     1650.0,     1660.0,     1670.0, 
     &  1680.0,     1690.0,     1700.0,     1710.0,     1720.0, 
     &  1730.0,     1740.0,     1750.0,     1760.0,     1770.0, 
     &  1780.0,     1790.0,     1800.0,     1810.0,     1820.0, 
     &  1830.0,     1840.0,     1850.0,     1860.0,     1870.0, 
     &  1880.0,     1890.0,     1900.0,     1910.0,     1920.0, 
     &  1930.0,     1940.0,     1950.0,     1960.0,     1970.0, 
     &  1980.0,     1990.0,     2000.0,     2010.0,     2020.0, 
     &  2030.0,     2040.0,     2050.0,     2060.0,     2070.0,
     &  2080.0,     2090.0,     2100.0,     2110.0,     2120.0, 
     &  2130.0,     2140.0,     2150.0,     2160.0,     2170.0, 
     &  2180.0,     2190.0,     2200.0,     2210.0,     2220.0, 
     &  2230.0,     2240.0,     2250.0,     2260.0,     2270.0, 
     &  2279.40330, 2290.0,     2300.0,     2310.0,     2320.0, 
     &  2330.0,     2340.0,     2350.0,     2360.0,     2370.0, 
     &  2380.0,     2390.0,     2400.0,     2410.0,     2420.0, 
     &  2430.0,     2440.0,     2450.0,     2460.0,     2470.0, 
     &  2480.0,     2490.0,     2500.0,     2510.0,     2520.0, 
     &  2530.0,     2540.0,     2550.0,     2560.0,     2570.0,
     &  2580.0,     2590.0,     2600.0,     2610.0,     2620.0,
     &  2630.0,     2640.0,     2650.0,     2660.0,     2670.0,
     &  2680.0,     2690.0,     2700.0,     2710.0,     2720.0,
     &  2730.0,     2740.0,     2750.0,     2760.0,     2770.0,
     &  2780.0,     2790.0,     2800.0,     2810.0,     2820.0,
     &  2830.0,     2840.0,     2850.0,     2860.0,     2870.0,
     &  2880.0,     2890.0,     2900.0,     2910.0,     2920.0,
     &  2930.0,     2940.0,     2950.0,     2960.0,     2970.0,
     &  2980.0,     2990.0,     3000.0,     3010.0,     3020.0,
     &  3030.0,     3040.0,     3050.0,     3060.0,     3070.0,
     &  3080.0,     3090.0,     3100.0,     3110.0,     3120.0, 
     &  3130.0,     3140.0,     3150.0,     3160.0,     3170.0, 
     &  3180.0,     3190.0,     3200.0,     3220.0,     3240.0, 
     &  3260.0,     3282.34820, 3300.0,     3320.0,     3340.0, 
     &  3360.0,     3380.0,     3400.0,     3420.0,     3440.0, 
     &  3460.0,     3480.0,     3500.0,     3520.0,     3540.0, 
     &  3560.0,     3580.0,     3600.0,     3620.0,     3640.0, 
     &  3660.0,     3680.0,     3700.0,     3720.0,     3740.0, 
     &  3760.0,     3780.0,     3800.0,     3820.0,     3840.0, 
     &  3860.0,     3880.0,     3900.0,     3920.0,     3940.0,
     &  3960.0,     3980.0,     4000.0,     4020.0,     4040.0, 
     &  4060.0,     4080.0,     4100.0,     4120.0,     4140.0, 
     &  4160.0,     4180.0,     4200.0,     4220.0,     4240.0, 
     &  4260.0,     4280.0,     4300.0,     4320.0,     4340.0, 
     &  4360.0,     4380.0,     4400.0,     4420.0,     4440.0, 
     &  4460.0,     4480.0,     4500.0,     4520.0,     4540.0, 
     &  4560.0,     4580.0,     4600.0,     4620.0,     4640.0, 
     &  4660.0,     4680.0,     4700.0,     4720.0,     4740.0, 
     &  4760.0,     4780.0,     4800.0,     4820.0,     4840.0, 
     &  4860.0,     4880.0,     4900.0,     4920.0,     4940.0,
     &  4960.0,     4980.0,     5000.0,     5020.0,     5040.0, 
     &  5060.0,     5080.0,     5100.0,     5120.0,     5140.0, 
     &  5160.0,     5180.0,     5200.0,     5220.0,     5240.0, 
     &  5260.0,     5280.0,     5300.0,     5320.0,     5340.0, 
     &  5360.0,     5380.0,     5400.0,     5420.0,     5440.0, 
     &  5460.0,     5480.0,     5500.0,     5520.0,     5540.0, 
     &  5560.0,     5580.0,     5600.0,     5620.0,     5640.0, 
     &  5660.0,     5680.0,     5700.0,     5720.0,     5740.0,    
     &  5760.0,     5780.0,     5800.0,     5820.0,     5840.0,    
     &  5860.0,     5880.0,     5900.0,     5920.0,     5940.0,
     &  5960.0,     5980.0,     6000.0,     6020.0,     6040.0,
     &  6060.0,     6080.0,     6100.0,     6120.0,     6140.0, 
     &  6160.0,     6180.0,     6200.0,     6220.0,     6240.0, 
     &  6260.0,     6280.0,     6300.0,     6320.0,     6340.0, 
     &  6360.0,     6380.0,     6400.0,     6440.0,     6480.0, 
     &  6520.0,     6560.0,     6600.0,     6640.0,     6680.0, 
     &  6720.0,     6760.0,     6800.0,     6840.0,     6880.0, 
     &  6920.0,     6960.0,     7000.0,     7040.0,     7080.0, 
     &  7120.0,     7160.0,     7200.0,     7240.0,     7280.0, 
     &  7320.0,     7360.0,     7400.0,     7440.0,     7480.0,
     &  7520.0,     7560.0,     7600.0,     7640.0,     7680.0, 
     &  7720.0,     7760.0,     7800.0,     7840.0,     7880.0, 
     &  7920.0,     7960.0,     8000.0,     8040.0,     8080.0, 
     &  8120.0,     8160.0,     8200.0,     8240.0,     8280.0, 
     &  8320.0,     8360.0,     8400.0,     8440.0,     8480.0, 
     &  8520.0,     8560.0,     8600.0,     8640.0,     8680.0, 
     &  8720.0,     8760.0,     8800.0,     8840.0,     8880.0, 
     &  8920.0,     8960.0,     9000.0,     9040.0,     9080.0, 
     &  9120.0,     9160.0,     9200.0,     9240.0,     9280.0, 
     &  9320.0,     9360.0,     9400.0,     9440.0,     9480.0,
     &  9520.0,     9560.0,     9600.0,     9640.0,     9680.0, 
     &  9720.0,     9760.0,     9800.0,     9840.0,     9880.0, 
     &  9920.0,     9960.0, 10000000.0/

      end module odf_tables

!***********************************************************************
